%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% ocrd_cluster.m
%
% OCRD_1.0 - One Class Rate Distortion based algorithm, Version 1.0
% Copyright (C) 2007 Koby Crammer, Partha Talukdar and Fernando Pereira
%
% See: 
%   A Rate-Distortion One-Class Model and its Applications to Clustering
%   Koby Crammer, Partha Pratim Talukdar and Fernando Pereira
%   Proceedings of the 25th International Conference on Machine Learning
%   (ICML), 2008 
%   http://www.cis.upenn.edu/~crammer/publications/icml08_ocrd.pdf
%
%    This program is free software; you can redistribute it and/or modify
%    it under the terms of the GNU General Public License as published by
%    the Free Software Foundation; either version 2 of the License, or
%    (at your option) any later version.
%
%    This program is distributed in the hope that it will be useful,
%    but WITHOUT ANY WARRANTY; without even the implied warranty of
%    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%    GNU General Public License for more details.
%
%    You should have received a copy of the GNU General Public License
%    along with this program; if not, write to the Free Software
%    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [res_s] = ocrd_cluster(data_s, params_s, verbose_s);
%
% data_s
% ------
%   x - data points to be analyzed. m column vectors of dimension n
%   p - prior importance (non negative, sums to one)
%   beta - inverse temperature
%
% params_s
% --------
%   q/mn - optional initialization. either a single cluster assignment or 
%          a centroid
%   bregman_fn - bregman function to be used
%   max_iters  - total number of iterations
%   thresh     - threshold for convergence
% 
% verbose_s
% ---------
%   verbose - reflect amount of output
%
% res_s
% -----  
%   q(i) = prob of point i to be in the single cluster 
%   q_1 = prob of the single cluster (its relative weight)
%   mn = centroid
%   d = divergence of input points from the current centroid
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  
x = data_s.data;
p = data_s.pdata;
beta = data_s.beta;
  
n = size(x, 1);
m = size(x, 2);
  
cont = 1;
round = 1;
    
    
% init
if isfield(params_s,'q')
    q = params_s.q;
    q_1 = sum(p.*q);      
    w = q .* p/q_1;
    mn = compute_mean(x, w);
elseif isfield(params_s,'mn')
    mn = params_s.mn;
    d = compute_dist(x, mn, params_s.bregman_fn);
    [q, q_1] = ocrd_fixed_d(d, p, beta);
else
    ind = floor(rand(1) * length(p))+1;
    mn = x(:, ind);
    d = compute_dist(x, mn, params_s.bregman_fn);
    [q, q_1] = ocrd_fixed_d(d, p, beta);
end  

q_old = q;

disp( sprintf('     round\tconvergence\tcluster size',round,max(abs(q - q_old)),sum(q==1) ));
while (cont)
    w = q .* p/q_1;
    mn = compute_mean(x, w);
    d = compute_dist(x, mn, params_s.bregman_fn);
    [q, q_1] = ocrd_fixed_d(d, p, beta);

    round = round + 1;

    
    if (verbose_s.level > 3),
      disp( sprintf('     %6d\t%e\t%d',round,max(abs(q - q_old)),sum(q==1) ));
    end

    if (verbose_s.level == 3),
      if mod(round, floor(params_s.max_rounds/20)) == 0
	disp( sprintf('     %6d\t%e\t%d',round,max(abs(q - q_old)),sum(q==1) ));
      end
    end

    
    
    if (max(abs(q-q_old)) < params_s.thresh)
        cont = 0;
    end
    
    if (round > params_s.max_rounds)
        cont = 0;
    end
    
    q_old = q;
end
disp( sprintf('done %6d\t%e\t%d',round,max(abs(q - q_old)),sum(q==1) ));
    
res_s.q = q;
res_s.q_1 = q_1;
res_s.mn = mn;
res_s.d = d;
  

