%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% ocrd_fixed_d.m
%
% OCRD_1.0 - One Class Rate Distortion based algorithm, Version 1.0
% Copyright (C) 2007 Koby Crammer, Partha Talukdar and Fernando Pereira
%
% See: 
%   A Rate-Distortion One-Class Model and its Applications to Clustering
%   Koby Crammer, Partha Pratim Talukdar and Fernando Pereira
%   Proceedings of the 25th International Conference on Machine Learning
%   (ICML), 2008 
%   http://www.cis.upenn.edu/~crammer/publications/icml08_ocrd.pdf
%
%    This program is free software; you can redistribute it and/or modify
%    it under the terms of the GNU General Public License as published by
%    the Free Software Foundation; either version 2 of the License, or
%    (at your option) any later version.
%
%    This program is distributed in the hope that it will be useful,
%    but WITHOUT ANY WARRANTY; without even the implied warranty of
%    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%    GNU General Public License for more details.
%
%    You should have received a copy of the GNU General Public License
%    along with this program; if not, write to the Free Software
%    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [q, q_1]=ocrd_fixed_d(d,p,beta,ub,nrm);
% 
% function [q, q_1]=ocrd_fixed_d(d,p,beta,ub,nrm,ks);
% compute the best probability assignments given
%   d - distances of points from centroid
%   p - prior weight
%   beta - inverse temperature
%   
%   ub - upper bound on q(i) ( should be 1)
%   nrm - final normalization in l_1 ( should be 1)
%
% return:
%   q - prob of point i to be in the cluster
%   q1 - prob of cluster
%
% see paper for details
%
  
  if ~exist('ub')
    ub =1 ;
  end
  
  
  if ~exist('nrm')
    nrm =1 ;
  end

  if ~exist('ks')
    ks = 0 ;
  end

  % Sort.
  [sd, si] = sort(beta .* d + log(p));



  % Initialization.
  m = size(p,1);
  c = m;
  a_curr = 1;
  p_curr = nrm; % was 1
  b_curr= beta * (p' * d);
  c_best = c;
  J = ones(m,1)*NaN;
  J(m) = b_curr;
  J_best = J(m);
  
  % Ind range from m:1. At each iteration,
  % we see whether the subset 1:(c-1) is better as an
  % one class cluster compared to the subset 1:c,
  % subject to satisfaction of constraints.
  
  while (c > 1)
    
    % Original index of x.
    oxi = si(c);    % smallest index out
    oxii = si(c-1); % max index in
    
    a_new = a_curr - exp(-1 * beta * d(oxi));
    p_new = p_curr - p(oxi);
    b_new = b_curr -  p(oxi) * (beta*d(oxi) + log(p(oxi)));
    
    q_1 = (p_new)/(a_new);
    q_in_min  = q_1 * exp(-beta * d(oxii)) / p(oxii);
    q_out_max = q_1 * exp(-beta * d(oxi))  / p(oxi);
    

    
    
    if ( (q_1>0) & (q_in_min > ub) & (q_out_max < ub))
      
      % Compute J_new
      J_new = b_new - p_new*log(p_new) + p_new * log(a_new);
      J(c-1) = J_new;
      
      if (J_new < J_best)
	c_best = c - 1;
	J_best = J_new;
      end;
    else
   %   fprintf('Constraint violated with OC cluster size:%d\n', (c-1));
    end;
    
    a_curr = a_new;
    p_curr = p_new;
    b_curr = b_new;
    c = c - 1;
  end;
  
  q = zeros(m,1);
  
  % never leave an empty ball
  if (1)
    q = zeros(m,1);
    if (c_best >= 1 && c_best <= m)
      inside = si(1:c_best);
      out_sum = 0;
      q_1 = sum(p(inside)) / (1 - out_sum);
      
      if (c_best < m)
	outside = si(c_best+1:m);
	out_sum = sum(exp(-1 * beta * d(outside)));
	q_1 = sum(p(inside)) / (1 - out_sum);
	q(outside) = q_1 * exp(-1 * beta * d(outside)) ./ p(outside);
      end
      
      q(inside) = ub;
    end;
  else
    q_1 = 0;
  end
  
  
  if (q>1)
    disp('error, q>1');
    keyboard;
  end
  